@@ -2,7 +2,7 @@
 //!
 use std::fmt::Display;
 
-use crate::collections::key_map::Handle;
+use crate::collections::handle_table::Handle;
 use crate::prelude::CardIndex;
 use crate::traits::VmFunction;
 use thiserror::Error;
@@ -50,11 +50,11 @@ pub enum ExecutionErrorPayload {
 #[derive(Debug, Clone, Error)]
 pub struct ExecutionError {
     pub payload: ExecutionErrorPayload,
-    pub trace: CardIndex,
+    pub trace: Vec<CardIndex>,
 }
 
 impl ExecutionError {
-    pub fn new(payload: ExecutionErrorPayload, trace: CardIndex) -> Self {
+    pub fn new(payload: ExecutionErrorPayload, trace: Vec<CardIndex>) -> Self {
         Self { payload, trace }
     }
 }
