@@ -4,7 +4,7 @@ use tracing::debug;
 
 use crate::{
     bytecode::{decode_str, read_from_bytes, TriviallyEncodable},
-    collections::key_map::Handle,
+    collections::handle_table::Handle,
     compiled_program::CaoCompiledProgram,
     procedures::ExecutionErrorPayload,
     traits::MAX_STR_LEN,
@@ -116,6 +116,7 @@ pub fn instr_string_literal<T>(
 }
 
 pub fn instr_jump(
+    src_ptr: usize,
     instr_ptr: &mut usize,
     program: &CaoCompiledProgram,
     runtime_data: &mut RuntimeData,
@@ -128,18 +129,19 @@ pub fn instr_jump(
         .call_stack
         .last_mut()
         .expect("Call stack was empty")
-        .instr_ptr = *instr_ptr;
+        .dst_instr_ptr = *instr_ptr as u32;
 
     // init the new call frame
     runtime_data
         .call_stack
         .push(CallFrame {
-            instr_ptr: *instr_ptr,
+            src_instr_ptr: src_ptr as u32,
+            dst_instr_ptr: *instr_ptr as u32,
             stack_offset: runtime_data
                 .value_stack
                 .len()
                 .checked_sub(argcount as usize)
-                .ok_or(ExecutionErrorPayload::MissingArgument)?,
+                .ok_or(ExecutionErrorPayload::MissingArgument)? as u32,
         })
         .map_err(|_| ExecutionErrorPayload::CallStackOverflow)?;
 
@@ -176,6 +178,7 @@ pub fn set_local<T>(vm: &mut Vm<T>, bytecode: &[u8], instr_ptr: &mut usize) -> E
         .last()
         .expect("Call stack is emtpy")
         .stack_offset;
+    let offset = offset as usize;
     let value = vm.runtime_data.value_stack.pop_w_offset(offset);
     vm.runtime_data
         .value_stack
@@ -192,8 +195,9 @@ pub fn get_local<T>(vm: &mut Vm<T>, bytecode: &[u8], instr_ptr: &mut usize) -> E
         .runtime_data
         .call_stack
         .last()
-        .expect("Call stack is emtpy")
+        .expect("Call stack is empty")
         .stack_offset;
+    let offset = offset as usize;
     let value = vm.runtime_data.value_stack.get(offset + handle as usize);
     vm.stack_push(value)?;
     Ok(())
@@ -203,7 +207,10 @@ pub fn instr_return<T>(vm: &mut Vm<T>, instr_ptr: &mut usize) -> ExecutionResult
     // pop the current stack frame
     let value = match vm.runtime_data.call_stack.pop() {
         // return value
-        Some(rt) => vm.runtime_data.value_stack.clear_until(rt.stack_offset),
+        Some(rt) => vm
+            .runtime_data
+            .value_stack
+            .clear_until(rt.stack_offset as usize),
         None => {
             return Err(ExecutionErrorPayload::BadReturn {
                 reason: "Call stack is empty".to_string(),
@@ -212,8 +219,10 @@ pub fn instr_return<T>(vm: &mut Vm<T>, instr_ptr: &mut usize) -> ExecutionResult
     };
     // read the previous frame
     match vm.runtime_data.call_stack.last_mut() {
-        Some(CallFrame { instr_ptr: ptr, .. }) => {
-            *instr_ptr = *ptr;
+        Some(CallFrame {
+            dst_instr_ptr: ptr, ..
+        }) => {
+            *instr_ptr = *ptr as usize;
         }
         None => {
             return Err(ExecutionErrorPayload::BadReturn {
@@ -305,7 +314,7 @@ pub fn for_each<T>(vm: &mut Vm<T>) -> Result<bool, ExecutionErrorPayload> {
 
     let should_continue = 0 <= i && i < n;
     if should_continue {
-        let key = obj.iter().nth(i as usize).map(|(k, _)| k).ok_or_else(|| {
+        let key = *obj.iter().nth(i as usize).map(|(k, _)| k).ok_or_else(|| {
             ExecutionErrorPayload::invalid_argument(format!(
                 "ForEach can not find the `i`th argument. i: {} n: {}\nDid you remove items during iteration?",
                 i, n
