@@ -63,11 +63,10 @@ fn test_trace_entry() {
     let trace = err.trace;
 
     let error_card = ir
-        .get_card(&trace)
+        .get_card(&trace[0])
         .expect("Expected to find the errored card");
 
     assert!(matches!(error_card, Card::CallNative(_)));
-    assert_eq!(trace.lane.as_str(), "pooh");
 }
 
 #[test]
@@ -667,12 +666,12 @@ fn len_test_happy() {
                 // same property as above
                 .with_card(Card::ReadVar(t.clone()))
                 .with_card(Card::StringLiteral(StringNode("asd".to_string())))
-                .with_card(Card::ScalarInt(IntegerNode(42)))
+                .with_card(Card::ScalarInt(IntegerNode(69)))
                 .with_card(Card::SetProperty)
                 // new property
                 .with_card(Card::ReadVar(t.clone()))
                 .with_card(Card::StringLiteral(StringNode("basdasd".to_string())))
-                .with_card(Card::ScalarInt(IntegerNode(42)))
+                .with_card(Card::ScalarInt(IntegerNode(89)))
                 .with_card(Card::SetProperty)
                 // len
                 .with_card(Card::ReadVar(t.clone()))
@@ -988,3 +987,66 @@ fn import_super_module_test() {
 
     assert_eq!(result, "poggers");
 }
+
+#[test]
+fn local_variable_regression_test() {
+    /*
+    In main:
+        In composite card:
+            set local var (1)
+
+    In other lane:
+        in composite card:
+            set another local var (2) (name can be different)
+        in another composite card:
+            read the local var
+
+    Bug: Reads the variable (1) from main
+    Should read variable (2)
+    */
+    let cu = Module {
+        imports: [].into(),
+        submodules: [].into(),
+        lanes: [
+            (
+                "main".into(),
+                Lane::default()
+                    .with_card(Card::composite_card(
+                        "asd",
+                        "asd",
+                        vec![Card::scalar_int(69), Card::set_var("pog").unwrap()],
+                    ))
+                    .with_card(Card::jump("tiggers")),
+            ),
+            (
+                "tiggers".into(),
+                Lane::default()
+                    .with_card(Card::composite_card(
+                        "basd",
+                        "basd",
+                        vec![Card::scalar_int(42), Card::set_var("foo").unwrap()],
+                    ))
+                    .with_card(Card::composite_card(
+                        "zasd",
+                        "zasd",
+                        vec![
+                            Card::read_var("foo").unwrap(),
+                            Card::set_global_var("pooh").unwrap(),
+                        ],
+                    )),
+            ),
+        ]
+        .into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("pooh", &program.variables)
+        .expect("Failed to read pooh variable");
+
+    assert_eq!(result.as_int().unwrap(), 42);
+}
